/* Hostloop -- the Hostloop Library
 * Copyright (C) 1998 Renaud Deraison
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * This library determines the names of the hosts that have special
 * relations with a first host (ie : they can mount its exported fs,
 * or they belong to the same (sub)network).
 * 
 * Used recursively, this library allow the test of a range of computers
 *
 *
 * Everything here is (c) 1998 Renaud Deraison <deraison@worldnet.fr>, except :
 *
 *  showmount.c : Copyright (c) 1993 Rick Sladkey <jrs@world.std.com> 
 *  host.c      : Copyright (c) 1986 The Regents of the University of California
 *                Portions Copyright (c) 1993 by Digital Equipment Corporation.
 *     
 *
 */
 
 
 
/* Includes */
#include <config.h>
#include <nessuslib.h>
#include <nessus-devel.h>
#include <stdlib.h>
#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "parser.h"
#include "hosts_loop.h"



struct hls_globals_t { 
 struct host_in_loop *hil;
 struct host_in_loop *c_hil;
 short num_hil;
 short cur_hil;
 struct domainlist_t * dml;
 };
 
struct hls_globals_t hls_globals;
	
/* struct host_in_loop *hil;
struct host_in_loop *c_hil = NULL;
short num_hil;
short cur_hil;
struct domainlist_t * dml = NULL;
 */



/* Local function */
static short is_host_present(char *, struct host_in_loop * , short);
void add_domain_hosts(struct host_in_loop * hosts, short host_id, short * num_hosts);
void add_showmount_hosts(struct host_in_loop * hosts, short host_id, short * num_hosts);
short is_host_tested(char * hostname, struct host_in_loop * hil, short num_hil);
static short is_host_present(char *hostname, struct host_in_loop * hil, short num_hil);
short is_domain_tested(char * domain, struct domainlist_t * dml);
static short is_ip_tested(char * hostname, struct host_in_loop * hil, 
             short num_hil);
/* void init_parses(void); */
int get_hostname_attributes(char * hostname);

/* Other functions */
extern char * name_to_domainname(char * hostname);
extern short get_host_rname(char ** hostname);
extern short get_domain_computers(char * domain, struct hostlist_t *hl);
extern int get_showmount_trusted(char * hostname, struct hostlist_t ** hostlist);  


void hls_new_session(char * hostname)
{
	hls_globals.hil = malloc(sizeof(struct host_in_loop));
	hls_globals.c_hil = NULL;
	hls_globals.dml = NULL;
	hls_globals.hil->hostname = malloc(strlen(hostname)+1);
	bzero(hls_globals.hil->hostname, strlen(hostname)+1);
	sprintf(hls_globals.hil->hostname, "%s", hostname);
	hls_globals.hil->id = 0;
	hls_globals.hil->trusted_by = -1;
	hls_globals.hil->tested = 0;
	hls_globals.hil->next = NULL;
	hls_globals.hil->addr = nn_resolve(hostname);
	hls_globals.num_hil = 1;
	hls_globals.cur_hil = 0;
	hls_globals.dml = NULL;  
	hls_globals.c_hil = hls_globals.hil;
}


char * hls_get_next_host()
{
	char * d;
	struct domainlist_t *ldml = hls_globals.dml;
	
	if(hls_globals.c_hil && hls_globals.c_hil->hostname)
		d=(char *)name_to_domainname(hls_globals.c_hil->hostname);
	else return(NULL);
	if(d && !is_domain_tested(d, hls_globals.dml))
	{
		
		add_domain_hosts(hls_globals.hil, hls_globals.cur_hil, &hls_globals.num_hil);
		if(!ldml)
			{
			hls_globals.dml = malloc(sizeof(struct domainlist_t));
			bzero(hls_globals.dml, sizeof(struct domainlist_t));
			hls_globals.dml->domain = malloc(strlen(d)+1);
			bzero(hls_globals.dml->domain, strlen(d)+1);
      			sprintf(hls_globals.dml->domain, "%s", d);
			}
		else
		{
		while(ldml->next)ldml = ldml->next;
		ldml->next = malloc(sizeof(struct domainlist_t));
		bzero(ldml->next, sizeof(struct domainlist_t));
		ldml = ldml->next;
		ldml->domain = malloc(strlen(d)+1);
		bzero(ldml->domain, strlen(d)+1);
		sprintf(ldml->domain, "%s", d);
		}
	}
	
	/* if(pSM(get_hostname_attributes(hls_globals.c_hil->hostname)))
	add_showmount_hosts(hls_globals.hil, hls_globals.cur_hil, &hls_globals.num_hil); */
	hls_globals.c_hil = hls_globals.hil;
skip:
	hls_globals.cur_hil++;
	if(hls_globals.cur_hil <= hls_globals.num_hil)
	{
	while(hls_globals.c_hil && hls_globals.c_hil->id != hls_globals.cur_hil)
		hls_globals.c_hil = hls_globals.c_hil->next;
	}
	else hls_globals.c_hil = NULL;
	/* if(c_hil && !pTEST(get_hostname_attributes(c_hil->hostname)))goto skip; */
	if(hls_globals.c_hil && 
	   is_host_tested(hls_globals.c_hil->hostname, 
	   hls_globals.hil, hls_globals.num_hil))
		goto skip;
	if(hls_globals.c_hil && 
		is_ip_tested(hls_globals.c_hil->hostname, 
		hls_globals.hil, hls_globals.num_hil))goto skip;
	if(hls_globals.c_hil)
		{
		get_host_rname(&hls_globals.c_hil->hostname);
		hls_globals.c_hil->tested = 1;
		}
	return(hls_globals.c_hil ? hls_globals.c_hil->hostname:NULL);
}
	
/*
  add_domain_hosts()

  Parameters :
     struct host_in_loop * hosts : a pointer to an already initialized
                                   host_in_loop structure.

     short host id     : the id number of the initial host of a domain
     short * num_hosts : a pointer to the current number of hosts in the
                         structure...

  Returns :
     nothing (void)
     *hosts content is modified
     num_hosts is modified
    
    This function adds all the hosts of the domain of the host <id> in the
    struct *hosts, checks that the same host is not put twice in the structure,
    caches the IP addresses of those hosts and gives them an id number
*/
void add_domain_hosts(struct host_in_loop * hosts, short host_id, short * num_hosts)
{
  short i;
  struct host_in_loop *hil_orig = hosts;
  struct hostlist_t * hl = malloc(sizeof(struct hostlist_t));
  char * hostname=NULL;
  char * domainname;
  
  hil_orig =  hosts;   /* save the starting point */
  
 
  /* Get the name of the host that we shall test 	*/
  while(hosts->id != host_id)
    {
      if(hosts && hosts->next)hosts = hosts->next;
      else 
	{
	  return;
	}
    }
  
  hostname = hosts->hostname;
  /* Now, we go to the end of the queue... */
  while(hosts->next)hosts=hosts->next;
  
  
  /* Now, let's get the list of the trusted hosts, and add it in the hil */
  
  if((domainname = (char *)name_to_domainname(hostname))==NULL)return;
  get_domain_computers(domainname, hl);
  if(hl->hostname)
    {
      struct hostlist_t * next;
      i = *num_hosts - 1;
      
      while(hl && hl->hostname)
	{
	 short do_test =1 ; /* pTEST(get_hostname_attributes(hl->hostname)); */
	 
	 
	  /* host_present() avoids to put twice the same host in
	     the hil */
	  
	
	 if(!is_host_present(hl->hostname, hil_orig, i) && do_test)
	    {
	      hosts->next = malloc(sizeof(struct host_in_loop));
	      bzero(hosts->next, sizeof(struct host_in_loop));
	      hosts = hosts->next;
	      hosts->hostname = malloc(strlen(hl->hostname)+1);
	      bzero(hosts->hostname, strlen(hl->hostname)+1);
	      strncpy(hosts->hostname, hl->hostname, strlen(hl->hostname));
	      hosts->hostip = malloc(strlen(inet_ntoa(hl->hostip)));
	      bzero(hosts->hostip, strlen(inet_ntoa(hl->hostip)));
	      strncpy(hosts->hostip, inet_ntoa(hl->hostip), 
		      strlen(inet_ntoa(hl->hostip)));
	      hosts->addr = hl->hostip;
	      hosts->id = ++i;
	      hosts->trusted_by = host_id;
	     /* hosts->full_results = malloc(sizeof(struct fullresults_t)); */
	    }
	  next = hl->next;
	  free(hl->hostname);
	  free(hl);
	  hl = next;
	}
    }
  else return;
  *num_hosts = i;
}

/*  add_showmount_hosts

    The same as add_domain_hosts() except that it adds the hosts which are allowed 
    to mount NFS partitions of the host <id> instead of adding the hosts of the
    same domain name.
*/
void add_showmount_hosts(struct host_in_loop * hosts, short host_id, 
			 short * num_hosts)
{
  short i;
  struct host_in_loop *hil_orig;
  struct hostlist_t * hl = malloc(sizeof(struct hostlist_t));
  char * hostname=NULL;

  bzero(hl, sizeof(struct hostlist_t));
  hil_orig =  hosts;   /* save the starting point */
  /* Get the name of the host that we shall test 	*/
  while(hosts->id != host_id)
    {
      if(hosts && hosts->next)hosts = hosts->next;
      else 
	{
	  return;
	}
    }
  hostname = hosts->hostname;
  /* Now, we go to the end of the queue... */
  while(hosts->next)hosts=hosts->next;
  /* Now, let's get the list of the trusted hosts, and add it in the hil */
  if(!get_showmount_trusted(hostname ,&hl ))
    {
      struct hostlist_t * next;
      i = *num_hosts - 1 ;
      while(hl && hl->hostname)
	{
	  char * tmp = "";
	  /* host_present() avoids to put twice the same host in
	     the hil */
	  /* tmp = name_to_ip(hl->hostname); */
	  
	  if(!is_host_present(hl->hostname, hil_orig, i))
	    {
	      hosts->next = malloc(sizeof(struct host_in_loop));
	      bzero(hosts->next, sizeof(struct host_in_loop));
	      hosts = hosts->next;
	      hosts->hostname = malloc(strlen(hl->hostname)+1);
	      bzero(hosts->hostname, strlen(hl->hostname)+1);
	      
	      strncpy(hosts->hostname, hl->hostname, strlen(hl->hostname));
	      hosts->hostip = malloc(strlen(tmp)+1);
	      bzero(hosts->hostip, strlen(tmp)+1);
	      strncpy(hosts->hostip, tmp, strlen(tmp));
	      hosts->addr = hl->hostip;
	      hosts->id = ++i;
	      hosts->trusted_by = host_id;
	    }
	  next = hl->next;
	  free(hl->hostname);
	  free(hl);
	  hl = next;
	}
    }
  else return;
  *num_hosts = i + 1;
}


/*   is_host_tested()

     Parameters :
     char * hostname : name of the host
     hil, num_hil : struct and number of items in struct

     Returns :
     1 if this host was tested
     0 if it was not
   
     This function determines wether a given host was already
     tested or not
*/

short is_host_tested(char * hostname, struct host_in_loop * hil, short num_hil)
{
  while(num_hil--)
    {
      if(!strcmp(hostname, hil->hostname) && hil->tested)return(1);
      else hil = hil->next;
    }
  
  return(0);
}
		
		
/* is_host_present()

   Same as is_host_tested but, returns :
   1 if the host is present in the hil structure or
   0 if it's not
*/

static short is_host_present(char *hostname, struct host_in_loop * hil, 
             short num_hil)
{	
  struct in_addr addr;
  
  addr = nn_resolve(hostname);
  
  while(num_hil--)
    {
      if(!strcmp(hostname, hil->hostname) ||
      	(addr.s_addr == hil->addr.s_addr))return(1);
      else hil = hil->next;
    }
  return(0);
	
}



/* is_ip_tested()

   Same as is_host_tested but, returns :
   1 if the host ip is present in the hil structure or
   0 if it's not
*/



static short is_ip_tested(char * hostname, struct host_in_loop * hil, 
             short num_hil)
{
	struct in_addr addr = nn_resolve(hostname);
	while(num_hil--)
	{
	 if((addr.s_addr == hil->addr.s_addr)&&hil->tested)
	  return(1);
	 hil=hil->next;
	}
 return(0);
}

/*	is_domain_tested()

	Determines wether a given domain has already been tested.
	Returns :
	   1 if it has been already tested
	   0 if it has not
*/

short is_domain_tested(char * domain, struct domainlist_t * dml)
{
  if(!domain)return(1);
  while(dml && dml->domain)
    {
      if(!strncasecmp(domain, dml->domain, strlen(domain)))return(1);
      else dml = dml->next;
    }
  return(0);
}
	
	
	
	
