/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <nessuslib.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include "parser.h"
#include <gtk/gtk.h>
#include "xstuff.h"
#include "auth.h"

int parse_message_type(type)
	char * type;
{
#ifdef DEBUGMORE
 fprintf(stderr, "%s:%d type : %s\n", __FILE__, __LINE__, type);
#endif
 if(!strncmp(MSG_HOLE_STR, type, strlen(MSG_HOLE_STR)))return(MSG_HOLE);
 if(!strncmp(MSG_INFO_STR, type, strlen(MSG_INFO_STR)))return(MSG_INFO);
 if(!strncmp(MSG_STAT_STR, type, strlen(MSG_STAT_STR)))return(MSG_STAT);
 if(!strncmp(MSG_PORT_STR, type, strlen(MSG_PORT_STR)))return(MSG_PORT);
 if(!strncmp(MSG_ERROR_STR, type, strlen(MSG_ERROR_STR)))return(MSG_ERROR);
 if(!strncmp(MSG_PING_STR, type, strlen(MSG_PING_STR)))return(MSG_PING);
 if(!strncmp(MSG_BYE_STR, type, strlen(MSG_BYE_STR)))return(MSG_BYE);
 return(-1);
}

int parse_server_message(message, hosts, humanmsg)
	char * message;
	struct arglist * hosts;
	char * humanmsg;
{
 char * message_type;
 char * buf;
 char * t;
 
 t = strstr(message, "SERVER <|> ");
 if(!t)return(-1);
 buf = strstr(message, "<|>");
 buf+=4;
 t = strstr(buf, "<|>");
 if(t)
 {
 t[0]=0;
 }
 message_type = emalloc(strlen(buf)+1);
 strncpy(message_type,buf, strlen(buf));
 if(t)t[0]='<';
 switch(parse_message_type(message_type))
 {
  case MSG_ERROR :
  	sprintf(humanmsg, "Error : %s", t);
  	return(MSG_ERROR);
  	break;
  case MSG_PORT :
  	parse_host_add_port(hosts, t, humanmsg);
  	return(MSG_PORT);
  	break;
  case MSG_HOLE :
     	humanmsg[0]=0;
  	parse_host_add_data(hosts, t, MSG_HOLE);
  	return(MSG_HOLE);
  	break;
  case MSG_INFO :
  	humanmsg[0]=0;
  	parse_host_add_data(hosts, t, MSG_INFO);
  	break;
  case MSG_STAT :
  	strncpy(humanmsg, buf+strlen(message_type), strlen(buf+strlen(message_type)));
  	return(MSG_STAT);
  	break;
  case MSG_PING :
  	humanmsg[0]=0;
  	auth_printf("CLIENT <|> PONG <|> PING <|> CLIENT\n");
  	return(MSG_PING);
  	break;
  case MSG_BYE :
        humanmsg[0]=0;
  	return(MSG_BYE);
  	break;
  }
 efree(message_type);
 return(-1);
}


void parse_nessusd_state(servmsg, host, current, max)
	char * servmsg;
	char * host;
	int  * current;
	int * max;
{
	char * t1,*t2;
	
	t1 = parse_separator(servmsg);if(!t1)return;
	t2 = parse_separator(servmsg+strlen(t1)+3);
	if(!t2)return;
	
	strncpy(host, t1, strlen(t1));
	t1 = strchr(t2, '/');
	t1[0]=0;
	*current = atoi(t2);
	t1+=sizeof(char);
	*max = atoi(t1);
}

void parse_host_add_port(hosts, servmsg, humanmsg)
	struct arglist * hosts;
	char * servmsg;
	char * humanmsg;
{
 char * port;
 char * hostname;
 char * t1, * t2;
 struct arglist * host;
 
 t1 = parse_separator(servmsg);if(!t1)return;
 t2 = parse_separator(servmsg+strlen(t1)+3);
 
#ifdef DEBUGMORE
 fprintf(stderr, "%s:%d t1: %s\n", __FILE__, __LINE__, t1);
#endif
 hostname = emalloc(strlen(t1)+1);
 strncpy(hostname, t1, strlen(t1));
 efree(t1);
 
#ifdef DEBUGMORE
 fprintf(stderr, "%s:%d t2: %s\n", __FILE__, __LINE__, t2);
#endif

 port = emalloc(strlen(t2)+1);
 strncpy(port, t2, strlen(t2));
 efree(t2);
 
 host = arg_get_value(hosts, hostname);
 if(!host)
 {
  host = emalloc(sizeof(struct arglist));
  arg_add_value(hosts, hostname, ARG_ARGLIST, -1, host);
 }
 
  host_add_port(host, atoi(port), 1, "");
#ifdef DEBUGMORE
  fprintf(stderr, "Port %s opened on %s\n", port, hostname);
#endif

  sprintf(humanmsg, "Port %s opened on %s", port, hostname);
  efree(hostname);
  efree(port);
}

void parse_host_add_data(hosts, servmsg, type)
	struct arglist * hosts;
	char * servmsg;
	int type;
{
 int port;
 char * data;
 char * hostname;
 char * t1, * t2, *t3;
 struct arglist * host;

 t1 = parse_separator(servmsg);if(!t1)return;
 t2 = parse_separator(servmsg+strlen(t1)+3);if(!t2){
 	      efree(t1);
 		return;
 		}
 
 hostname = emalloc(strlen(t1)+1);
 strncpy(hostname, t1, strlen(t1));
 efree(t1);
 
 t3 = strchr(t2, ':');
 if(t3){
 	t3[0]=0;
 	t3++;
 	}
 else {
 	efree(t1);
 	efree(t2);
 	return;
 	}
 
 port = atoi(t2);
 data = emalloc(strlen(t3)+1);
 strncpy(data, t3, strlen(t3));
 efree(t2);
 while((t2=strchr(data, ';')))t2[0]='\n';
 
 host = arg_get_value(hosts, hostname);
 if(!host)
 {
  host = emalloc(sizeof(struct arglist));
  arg_add_value(hosts, hostname, ARG_ARGLIST, -1, host);
 }
 
  switch(type)
  {
   case MSG_HOLE :
   l_post_hole(host, port, data);
   break;
   case MSG_INFO :
   l_post_info(host, port, data);
   break;
   default :
   fprintf(stderr, "received unknown message type (%d)\n", type);
   break;
  }
#ifdef DEBUGMORE
  fprintf(stderr,"data for %s (port %d) [type : %d] : \n%s\n", hostname, port, type,data);
#endif
 
  efree(hostname);
  efree(data);
}
 

 
char * parse_symbol(str)
	char * str;
{
 char * s = str;
 
 while(s)
 {
#ifdef DEBUGMORE
 fprintf(stderr, "%s:%d s : %s\n", __FILE__, __LINE__, s);
#endif
 s = strchr(s, '|');
 if(!s)return(NULL);
 if((s[1]=='>')&&(s-1)[0]=='<')
 	{
#ifdef DEBUGMORE
 	fprintf(stderr, "returning \"%s\"\n", s+3);
#endif
 	return(s+3);
 	}
 s++;
 }
 return(NULL);
}

char * parse_separator(str)
	char * str;
{
 char * s_1, *s_2;
 char * ret;
 
 s_1 = parse_symbol(str);
 if(!s_1)return(NULL);
 s_2 = parse_symbol(s_1);
 if(!s_2)
  {
  ret = emalloc(strlen(s_1));
  strncpy(ret, s_1, strlen(s_1)-1);
  }
 else
 {
 int c;
 s_2 = s_2 - 4;
 c = s_2[0];
 s_2[0] = 0;
 ret = emalloc(strlen(s_1));
 strncpy(ret, s_1, strlen(s_1)-1);
 s_2[0]=c;
 }
#ifdef DEBUGMORE
 fprintf(stderr, "%s:%d gpt %s returning \"%s\"\n", __FILE__, __LINE__, str, ret);
#endif
  return(ret);
}
 
 
