/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */  
 
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <nessuslib.h>
#include <nessus-devel.h>
#include <gtk/gtk.h>
#include "report.h"
#include "xstuff.h"




struct report_data{
  GtkWidget *report_window;
  void *vdata;
  long ldata;
  struct arglist * hosts;
  GtkWidget * hosts_list;
  GtkWidget * ports_list;
  GtkWidget * holes_list;
  GtkWidget * hosts_win;
  GtkWidget * ports_win;
  GtkWidget * holes_win;
};


extern struct arglist * hosts;

void fill_hosts(struct arglist * , struct report_data * );
void hosts_callback(GtkWidget * , struct report_data * );
void ports_callback(GtkWidget * list, struct report_data * data);
void update_ports(struct arglist * , char * , struct report_data * );

/*
 * Creation of the reporting window
 */

struct report_data *create_report_window(struct arglist * args)
{
  struct report_data * ret = emalloc(sizeof(struct report_data));
  GtkWidget * table;
  GtkWidget * label;

  ret->hosts = args;
  ret->report_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(ret->report_window), "destroy",
		     (GtkSignalFunc)close_window,ret->report_window);
  
  gtk_signal_connect(GTK_OBJECT(ret->report_window), "delete_event",
		     (GtkSignalFunc)close_window,ret->report_window);
  
  gtk_window_set_title(GTK_WINDOW(ret->report_window), "Nessus Report");
  gtk_container_border_width(GTK_CONTAINER(ret->report_window), 10);     
  
  
  table = gtk_table_new(20,6,TRUE);
  gtk_table_set_row_spacing(GTK_TABLE(table),0,15);
  gtk_table_set_col_spacings(GTK_TABLE(table),5);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(ret->report_window), table);
  label = gtk_label_new("Hosts");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,2,0,1);
  gtk_widget_show(label);
  
  
  ret->hosts_win = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ret->hosts_win),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_table_attach_defaults(GTK_TABLE(table), ret->hosts_win, 0,2,1,20);
  gtk_widget_show(ret->hosts_win);
  
  ret->hosts_list = gtk_list_new();
  gtk_container_add(GTK_CONTAINER(ret->hosts_win), ret->hosts_list);
  
  gtk_signal_connect(GTK_OBJECT(ret->hosts_list),
		     "selection_changed",
		     GTK_SIGNAL_FUNC(hosts_callback),
		     ret);
  gtk_widget_show(ret->hosts_list);
  
  
  label = gtk_label_new("Ports");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 2,3,0,1);
  gtk_widget_show(label);
  
  ret->ports_win = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ret->ports_win),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_table_attach_defaults(GTK_TABLE(table), ret->ports_win, 2,3,1,20);
  gtk_widget_show(ret->ports_win);
  
  ret->ports_list = gtk_list_new();
  gtk_container_add(GTK_CONTAINER(ret->ports_win), ret->ports_list);
  gtk_signal_connect(GTK_OBJECT(ret->ports_list),
		     "selection_changed",
		     GTK_SIGNAL_FUNC(ports_callback),
		     ret);
  gtk_widget_show(ret->ports_list);
  
 
  
  label = gtk_label_new("Vulnerabilities");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 3,6,0,1);
  gtk_widget_show(label);
  
  ret->holes_win = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ret->holes_win),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_table_attach_defaults(GTK_TABLE(table), ret->holes_win, 3,6,1,20);
  gtk_widget_show(ret->holes_win); 
  
  ret->holes_list = gtk_list_new();
  gtk_container_add(GTK_CONTAINER(ret->holes_win), ret->holes_list);
  gtk_widget_show(ret->holes_list);
  gtk_widget_show(ret->report_window);
 
  return(ret);
}

/*
 * This function puts the names of the hosts tested in the first column (aka 
 * browser) of the window created by the function above
 */
void fill_hosts(struct arglist * hosts, struct report_data * data)
{
  int vul=0;
  GList *dlist = NULL;
  
  while(hosts && hosts->next)
    {
      struct arglist * ports;
      
      ports = arg_get_value(hosts->value, "PORTS");
#ifdef DEBUG
      if(!ports)fprintf(stderr, "%s:%d : ports == NULL\n", __FILE__, __LINE__);
#endif
      while(ports && ports->next && vul!=2)
	{
	  struct arglist * reports;
	  struct arglist * infos;
	  
	  infos = arg_get_value(ports->value, "INFO");
	  reports = arg_get_value(ports->value, "REPORT");
	  if((int)(arg_get_value(ports->value, "STATE")) == 1)
	    {
	      if(infos)vul=1;
	      if(reports)vul=2;
	    }
	  ports = ports->next;
	}
      if(vul)
	{
	  GtkWidget * list_item;
	  char * name = emalloc(strlen(hosts->name)+5);
	  if(vul==1)sprintf(name, "---! %s", hosts->name);
	  if(vul>1)sprintf(name, "---[ %s", hosts->name);
	  list_item = gtk_list_item_new_with_label(name);
          dlist = g_list_prepend(dlist, list_item);
          gtk_object_set_data(GTK_OBJECT(list_item),
			      "list_item_data",
			      hosts->name);
          gtk_widget_show(list_item);
	}
      else
        {
	  GtkWidget * list_item;
	  
	  list_item = gtk_list_item_new_with_label(hosts->name);
	  dlist = g_list_prepend(dlist, list_item);
	  gtk_object_set_data(GTK_OBJECT(list_item),
			      "list_item_data",
			      hosts->name);
	  gtk_widget_show(list_item);
	}
      hosts = hosts->next;
      vul = 0;
    }
  gtk_list_append_items(GTK_LIST(data->hosts_list), dlist);
}

/*
 * This function updates the portlist of the host which is currently clicked
 */
void update_ports(hosts,cur_host,data)
     struct arglist * hosts;
     char * cur_host;
     struct report_data * data;
{
  struct arglist * ports;
  GList * dlist;
  /*
   * Seek for the host that interests us ...
   */
  while(hosts && hosts->next && strcmp(hosts->name, cur_host))
    hosts = hosts->next;
  
  dlist = GTK_LIST(data->ports_list)->children;
  if(dlist)gtk_list_remove_items(GTK_LIST(data->ports_list),dlist);
  
  dlist = GTK_LIST(data->holes_list)->children;
  if(dlist)gtk_list_remove_items(GTK_LIST(data->holes_list), dlist);
  
  if(!hosts){
    fprintf(stderr, "Error ! Host selected not found !\n");
    return;
  }
  dlist = NULL;
  ports = arg_get_value(hosts->value, "PORTS");
  if(!ports)
    {
      /*
       * A given host MAY have NO port open, thus having a NULL
       * portlist is NOT an error
       */
      return;
    }
  else while(ports && ports->next)
    {
      struct arglist * reports;
      struct arglist * infos;
      
      reports = arg_get_value(ports->value, "REPORT");
      infos = arg_get_value(ports->value, "INFO");
      
      if((int)(arg_get_value(ports->value, "STATE")) == 1)
	{
	  /*
	   * If a vulnerability has been found on a port, then
	   * we write its number in bold, so that the user sees
	   * immediatly what is interesting...
	   */
	  if(reports || infos)
	    {
	      char * name = emalloc(strlen(ports->name)+6);
	      GtkWidget * list_item;
	      
	      if(reports)sprintf(name, "---[ %s", ports->name);
	      else sprintf(name, "---! %s", ports->name);
	      list_item = gtk_list_item_new_with_label(name);
	      gtk_widget_show(list_item);
	      dlist = g_list_append(dlist, list_item);
	      gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data",
				  ports->name);
	    }
	  else {
	    GtkWidget * list_item;
	    list_item = gtk_list_item_new_with_label(ports->name);
	    dlist = g_list_append(dlist, list_item);
	    gtk_widget_show(list_item);
	    gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data",
				ports->name);
	  }
	}
      ports = ports->next;
    }
  if(dlist)gtk_list_append_items(GTK_LIST(data->ports_list), dlist);
}

/*
 * This function writes the holes found for a given port
 */
void update_holes(hosts,cur_host,cur_port, data)
     struct arglist * hosts;
     char * cur_host;
     char * cur_port;
     struct report_data * data;
{
  struct arglist * ports;
  struct arglist * reports;
  struct arglist * infos;
  GList * dlist;
  
  dlist = GTK_LIST(data->holes_list)->children;
  if(dlist)gtk_list_remove_items(GTK_LIST(data->holes_list),dlist);
 
	
  /* 
   * Search for the host that interests us
   */
       while(hosts && hosts->next && 
             strcmp(hosts->name, cur_host))
         hosts = hosts->next;
       
	/*
        * Doh ! This should NOT happen
        */
       if(!hosts){
         fprintf(stderr, "Error ! Host selected not found !\n");
         return;
       }
       
       ports = arg_get_value(hosts->value, "PORTS");
       if(!ports){
         fprintf(stderr, "Error ! NULL portlist !\n");
         return;
       }
       
       while(ports && strcmp(ports->name, cur_port))
         ports = ports->next;
       
       if(!ports){
         fprintf(stderr, "Error ! Port selected not found !\n");
         return;
       }
       
       reports = arg_get_value(ports->value, "REPORT");
       dlist = NULL;
       while(reports && reports->next)
         {
          GtkWidget * list_item;
          char * t, * oldt;
          
          t = reports->value;
          oldt = t;
          while((t=strchr(t, '\n')))
          {
          t[0]=0;
          list_item = gtk_list_item_new_with_label(oldt);
          dlist = g_list_append(dlist, list_item);
          gtk_container_add(GTK_CONTAINER(data->holes_win), list_item);
          gtk_widget_show(list_item);
          gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data",
             			    reports->value);
          t[0]='\n';
          oldt = ++t;
          }
          list_item = gtk_list_item_new_with_label(oldt);
          dlist = g_list_append(dlist, list_item);
          gtk_container_add(GTK_CONTAINER(data->holes_win), list_item);
          gtk_widget_show(list_item);
          gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data",
             			    reports->value);
          list_item = gtk_list_item_new_with_label("\n");
          gtk_widget_show(list_item);
          dlist = g_list_append(dlist, list_item);
          reports = reports->next;
          
         }
         
        infos = arg_get_value(ports->value, "INFO");
        while(infos && infos->next)
        {
          GtkWidget * list_item;
          char * t,*oldt;
          
          t = infos->value;
          oldt = t;
      
          while((t=strchr(t, '\n')))
          {
          t[0]=0;
          list_item = gtk_list_item_new_with_label(oldt);
          dlist = g_list_append(dlist, list_item);
          gtk_container_add(GTK_CONTAINER(data->holes_win), list_item);
          gtk_widget_show(list_item);
          gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data",
             			    infos->value);
          t[0]='\n';
          oldt = ++t;
          }
          list_item = gtk_list_item_new_with_label(oldt);
          dlist = g_list_append(dlist, list_item);
          gtk_container_add(GTK_CONTAINER(data->holes_win), list_item);
          gtk_widget_show(list_item);
          gtk_object_set_data(GTK_OBJECT(list_item), "list_item_data",
             			    infos->value);
            list_item = gtk_list_item_new_with_label("\n");
          gtk_widget_show(list_item);
          dlist = g_list_append(dlist, list_item);  			
          infos = infos->next;
          
         }
   if(dlist)gtk_list_append_items(GTK_LIST(data->holes_list), dlist);
}

void hosts_callback(GtkWidget * list, struct report_data * data)
{
  char * cur_host = NULL;
  GList * dlist;
  GtkObject       *list_item;
  
  dlist = GTK_LIST(list)->selection;
  if(!dlist)return;
  list_item=GTK_OBJECT(dlist->data);  
  cur_host=gtk_object_get_data(list_item, "list_item_data");       
  update_ports(data->hosts, cur_host, data);
}


void ports_callback(GtkWidget * list, struct report_data * data)
{
  char * cur_host = NULL;
  char * cur_port = NULL;
  GList * hlist,*plist;
  GtkObject * list_item;
  
  hlist = GTK_LIST(data->hosts_list)->selection;
  if(!hlist)fprintf(stderr, "no current host ? \n");
  list_item = GTK_OBJECT(hlist->data);
  cur_host = gtk_object_get_data(list_item, "list_item_data");   
  
  
  plist = GTK_LIST(list)->selection;
  if(!plist)return;
  
  list_item = GTK_OBJECT(plist->data);
  cur_port = gtk_object_get_data(list_item, "list_item_data");
  
  update_holes(data->hosts, cur_host,cur_port, data);
}




/*
 *  Main function of the holes reporter
 */
void report_tests(struct arglist * hosts)
{
  struct report_data * data;
  data = create_report_window(hosts);
  fill_hosts(hosts, data);
  gtk_main();
}
