/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Nessus Communication Manager -- it manages the NTP Protocol, version 1.0
 *
 */ 
 
#include <config.h>
#include <nessus-devel.h>
#include <nessuslib.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include "auth.h"
#include "comm.h" 
#include "threads.h"

extern struct arglist * plugins;


/*
 * Once we have a socket, we must initiate the
 * connection and setup the program
 */
void comm_init(soc)
     int soc;
{  
  /* stdin and stdout == soc */
  close(0);dup(soc);
  close(1);dup(soc);
  /* This is **NOT** auth_printf, since nothing was negociated yet ! */
  printf("< NTP/1.0 >\n");
}


/*
 * This function must be called at the end
 * of a session
 */
void comm_terminate()
{
  auth_printf("SERVER <|> BYE <|> BYE <|> SERVER\n");
}


/*
 * Sends the list of plugins that the server
 * could load to the client, using the 
 * NTP format
 */
void comm_send_pluginlist()
{
  struct arglist * l_plug = plugins;
  int i=1,j;
  char * categories[] = {"infos","attack","denial","unknown"};
  
  auth_printf("SERVER <|> PLUGIN_LIST <|>\n");
  while(l_plug && l_plug->next)
    {
      struct arglist * args;
      
      args = arg_get_value(l_plug->value, "plugin_args");
      j = (int)arg_get_value(args, "CATEGORY");
      if((j>ACT_DENIAL) || (j<ACT_GATHER_INFO))j=4;
      auth_printf("%d <|> %s <|> %s <|> %s <|> %s <|> %s <|> %s\n", i, 
		  arg_get_value(args, "NAME"),
		  categories[j-1],
		  arg_get_value(args, "COPYRIGHT"),
		  arg_get_value(args, "DESCRIPTION"),
		  arg_get_value(args, "SUMMARY"),
		  plug_get_family(args));
      i++;
      l_plug = l_plug->next;
    }
  auth_printf("<|> SERVER\n");
}


/*
 * This function waits for the attack order
 * of the client
 */
char * comm_wait_order()
{
  char * str = emalloc(512);
  char * strs[7];
  int good=0;
  int i;
  
  while(!good)
    {
      auth_gets(str, 511);
      if(!strlen(str))continue;
#ifdef DEBUGMORE
      printf("Received : %s\n", str);
#endif
      for(i=0;i<7;i++)strs[i]=emalloc(strlen(str)+1);
      sscanf(str, "%s <|> %s <|> %s <|> %s <|> %s  <|> %s <|> %s",
	     strs[0],strs[1],strs[2],strs[3],strs[4],strs[5],strs[6]);
      
#ifdef DEBUGMORE
      for(i=0;i<7;i++)
	{
	  printf("strs[%d] : %s\n", i, strs[i]);
	}
#endif
      if(!strncmp(strs[1], "QUIT", 4)){
        for(i=0;i<7;i++)efree(strs[i]);
    	shutdown(0,2);
    	exit(0);
      }
      if(!strncmp(strs[0], "CLIENT", 6) && !strncmp(strs[1],"NEW_ATTACK",10) &&
	 !strncmp(strs[6], "CLIENT", 6))good = 1;
    }
  
  comm_setup_plugins(strs[2]);
  /*comm_setup_hostloop(strs[3], strs[4]);*/
  for(i=0;i<5;i++)efree(strs[i]);
  efree(strs[6]);
  efree(str);
  return(strs[5]);
}


/* 
 * Marks the plugins that will be
 * launched, according to the list
 * provided by the client
 */
void comm_setup_plugins(id_list)
     char * id_list;
{
  char * list = id_list;
  char * buf;
  int id;
  int i = 0;
  int num_plugins=0;
  struct arglist * l_plugin;
  /* list = a string like '16;5;3' */
  
  l_plugin = plugins;
  while(l_plugin && l_plugin->next){
    num_plugins++;
    l_plugin = l_plugin->next;
  }
  buf = emalloc(strlen(id_list)+1);
  while(sscanf(list, "%d;%s", &id, buf) && strlen(list))
    {
      l_plugin = plugins;
      if(id<=0)
 	{
	  struct arglist * lp = plugins;
	  while(lp && lp->next)
	    {
	      plug_set_launch(lp->value, 1);
	      lp = lp->next;
	    }
	  break;
 	}
      if(id>num_plugins)
	{
	  auth_printf("SERVER <|> ERROR <|> There's no plugin of id %d <|> SERVER\n", id);
	  l_plugin = NULL;
	}
      else for(i=1;i!=id;i++)
	{
	  l_plugin = l_plugin->next;
	}
      
      if(l_plugin)plug_set_launch(l_plugin->value, 1);
      sprintf(list, "%s", buf);
      bzero(buf, strlen(id_list)+1);
    }
}

void client_handler()
{
 char * buf = emalloc(400);
 if(!auth_gets_nonblock(buf, 399))
  { 
  char * t;
  if((t = strstr(buf, "STOP_ATTACK")))
  {
   char * t2;
   t2 = strstr(t, "<|> ");
   if(!t2)return;
   t2+=strlen("<|> ");
   t = strstr(t2, " <|>");
   if(!t)return;
   t[0] = 0;
   nessusd_thread_kill_by_name(t2);
   t[0] = ' ';
  }
 }
 efree(buf);
}
 
