/* Nessuslib -- the Nessus Library
 * Copyright (C) 1998 Renaud Deraison
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Plugutils -- manages the arglists and some plugin-specific stuff
 */

#include <config.h>
#include <nessuslib.h>
#include <nessus-devel.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */ 

void arg_add_value(struct arglist * arglst, char * name, int type, long length, void * value)
{
	
	if(!arglst)return;
	
	/* Go to the end of our list... */
	while(arglst->next)arglst = arglst->next;

	arglst->name = emalloc(estrlen(name, 255)+1);
	
	strncpy(arglst->name, name, estrlen(name, 255));
	arglst->value = value;
	
	arglst->length = length;
	arglst->type = type;
	arglst->next = emalloc(sizeof(struct arglist));
	bzero(arglst->next, sizeof(struct arglist));
	arglst = arglst->next;
	arglst->next = NULL;
}

int arg_set_value(struct arglist * arglst, char * name, long length, void *value)
{
  /* We look for the asked name */
  while(arglst->next && strncmp(arglst->name, name, strlen(name)))
    arglst = arglst->next;
  
  if(arglst->next)
    {
      arglst->value = value;
      return(0);
    }
  else return(-1);
  
}

int arg_set_type(struct arglist * arglst, char * name, int type)
{
  while(arglst->next && strncmp(name, arglst->name, strlen(name)))arglst=arglst->next;
  if(!(arglst->next))return(-1);
  else arglst->type = type;
  return 0;
}
  
void * arg_get_value(struct arglist * args, char * name)
{
 int flag=0;
 if(!args)return(NULL); 
  while(args && args->next && !flag)
  	{
        if(args->name && !strcmp(name, args->name))flag=1;
  	else args=args->next;
  	}
  if(args->next)
    return(args->value);
  else return(NULL);
}


int arg_get_type(struct arglist * args, char * name)
{
  while(args->next && strncmp(name, args->name, strlen(name)))args=args->next;
  if(args->next)
    return(args->type);
  else return(-1);
}


void arg_dump(struct arglist * args, int level)
{
	char * spaces = "--------------------";
	if(!args)
	{
		printf("Error ! args == NULL\n");
		return;
	}
	
	while(args && args->next)
	{
		switch(args->type)
		{
			case ARG_STRING :
			
			fprintf(stderr, "%sargs->%s : %s\n",spaces+(20-level),
			args->name,
			(char *)args->value);
			break;
			case ARG_ARGLIST :
			
			fprintf(stderr, "%sargs->%s :\n", spaces+(20-level),
			args->name);
			arg_dump(args->value, level+1);
			break;
			case ARG_INT :
			fprintf(stderr, "%sargs->%s : %d\n",spaces+(20-level),
			args->name,
			(int)args->value);
			break;
			default :
			fprintf(stderr, "%sargs->%s : %d\n",spaces+(20-level),
			args->name,
			(int)args->value);
			break;
		}
		args = args->next;
	}
}
			
		
void plug_set_family(struct arglist * desc, char * family)
{
  char * s = emalloc(strlen(family)+1);
  strncpy(s, family, strlen(family));
  arg_add_value(desc, "FAMILY", ARG_STRING, strlen(family), family);
}

char * plug_get_family(struct arglist * desc)
{
 return(arg_get_value(desc, "FAMILY"));
}
		
void plug_set_launch(struct arglist * desc, int launch)
{
  if(arg_set_value(desc, "ENABLED", sizeof(int), (void *)launch)<0)
   arg_add_value(desc, "ENABLED", ARG_INT, sizeof(int), (void*)launch);
}

int plug_get_launch(struct arglist * desc)
{
 int ret = (int)arg_get_value(desc, "ENABLED");
 if(ret<0)ret=1;
 return((int)arg_get_value(desc, "ENABLED"));
}	
	
void plug_set_name(struct arglist * desc, char * name) 
{	
  char * pname;
  pname = emalloc(estrlen(name, 255)+1);
  strncpy(pname, name, estrlen(name, 255));
  arg_add_value(desc, "NAME", ARG_STRING, strlen(name), pname);
}

void plug_set_summary(struct arglist * desc, char * summary)
{
  char * psummary;
  
  psummary = emalloc(estrlen(summary, 255)+1);
  strncpy(psummary, summary, estrlen(summary, 255));
  arg_add_value(desc, "SUMMARY", ARG_STRING, strlen(summary), psummary);
}


void plug_set_description(struct arglist * desc, char * description)
{
  char * pdescription = emalloc(estrlen(description, 255)+1);
  strncpy(pdescription, description, estrlen(description, 255));
  arg_add_value(desc, "DESCRIPTION", ARG_STRING, strlen(pdescription), pdescription);
}

void plug_set_copyright(struct arglist * desc, char * copyright)
{
	char * pcopyright = emalloc(estrlen(copyright, 255)+1);
	strncpy(pcopyright, copyright, estrlen(copyright, 255));
	arg_add_value(desc, "COPYRIGHT", ARG_STRING, strlen(pcopyright), pcopyright);
}

void plug_set_category(struct arglist * desc, int category)
{
       arg_add_value(desc, "CATEGORY", ARG_INT, sizeof(int), (void *)category);
}


void plug_set_attacklevel(struct arglist * desc, int attacklevel)
{
	arg_add_value(desc, "ATTACKLEVEL", ARG_INT, sizeof(int), (void *)attacklevel);
}

void plug_set_local_remote(struct arglist * desc, char local, char remote)
{
  long llocal = local;
  long lremote = remote;
  arg_add_value(desc, "LOCAL", ARG_INT, sizeof(char), (void *)llocal);
  arg_add_value(desc, "REMOTE", ARG_INT, sizeof(char), (void *)lremote);
}



void plug_add_targetos(struct arglist * desc, char * os_name)
{
  char * posname; /* = emalloc(estrlen(os_name,255)+1); */
  struct arglist * os;
 
  return;
  
  strncpy(posname, os_name, estrlen(os_name, 255));
  os = arg_get_value(desc, "OSNAMES");
  if(!os)
  {
  os = emalloc(sizeof(struct arglist));
  bzero(os, sizeof(struct arglist));
  arg_add_value(desc, "OSNAMES", ARG_ARGLIST, sizeof(struct arglist), os);
  }
  
  while(os && os->next)os = os->next;
  os->name = posname;
  os->next = emalloc(sizeof(struct arglist));
  bzero(os->next, sizeof(struct arglist));
}

void plug_add_targetos_family(struct arglist * desc, char * os_family)
{
  char * posfamily; /*  = emalloc(estrlen(os_family,255)+1); */
  struct arglist * os;
  
  return;
  strncpy(posfamily, os_family, estrlen(os_family,255));
  os = arg_get_value(desc, "OSFAMILIES");
  if(!os)
  {
  os = emalloc(sizeof(struct arglist));
  bzero(os, sizeof(struct arglist));
  arg_add_value(desc, "OSFAMILIES", ARG_ARGLIST, sizeof(struct arglist), os);
  }
  while(os && os->next)os = os->next;
  os->name = posfamily;
  os->next = emalloc(sizeof(struct arglist));
  bzero(os->next, sizeof(struct arglist));
}



struct arglist * plug_get_targetos(struct arglist * desc)
{
	return(arg_get_value(desc, "OSNAMES"));
}

struct arglist * plug_get_targetos_family(struct arglist * desc)
{
	return(arg_get_value(desc, "OSFAMILIES"));
}


void plug_add_host(struct arglist * desc, struct arglist * hostname)
{
	struct arglist * h;
	
	h = arg_get_value(desc, "HOSTNAME");
	if(!h)arg_add_value(desc, "HOSTNAME", ARG_ARGLIST, sizeof(hostname), hostname);
	else arg_set_value(desc, "HOSTNAME", sizeof(hostname), hostname);
}



void host_add_port(struct arglist * hostdata, int portnum, int state, char * banner)
{
 struct arglist * port ;
 struct arglist * portlist;
 char * port_s = emalloc(8);
 char * l_banner = emalloc(banner ? estrlen(banner,255)+1:0);
 sprintf(port_s, "%d", portnum);
 
 if(banner)strncpy(l_banner, banner, estrlen(banner,255));
 port = emalloc(sizeof(struct arglist));
 if(hostdata)
 	 portlist = arg_get_value(hostdata, "PORTS");
 else return;
 if(!portlist)
 {
#ifdef DEBUGMORE
  printf("!portlist\n");
  printf("allocating a new one...");
#endif
  portlist = emalloc(sizeof(struct arglist));
  arg_add_value(hostdata, "PORTS", ARG_ARGLIST, sizeof(portlist), portlist);
#ifdef DEBUGMORE
  printf("Done!\n");
#endif
 }

 arg_add_value(port, "STATE", ARG_INT, sizeof(int), (void *)state);
 arg_add_value(port, "BANNER", ARG_STRING, banner ? strlen(l_banner):0, l_banner);
 arg_add_value(portlist, port_s, ARG_ARGLIST, sizeof(port), port);
#ifdef DEBUGMORE
 printf("Port added for %s\n", (char *)arg_get_value(hostdata, "FQDN"));
#endif
}

int host_get_port_state(struct arglist * plugdata, int portnum)
{ 
 struct arglist * hostdata = arg_get_value(plugdata, "HOSTNAME");
 struct arglist * ports;
 struct arglist * port;
 char * port_s = emalloc(8);
 
 sprintf(port_s, "%d", portnum);
 if(!hostdata)return(0);
 ports = arg_get_value(hostdata, "PORTS");
 if(!ports)return(0);
 port = arg_get_value(ports, port_s);
 if(!port)return(0);
 return((int)arg_get_value(port, "STATE"));
}


char * host_get_port_banner(struct arglist * plugdata, int portnum)
{
  struct arglist * hostdata = arg_get_value(plugdata, "HOSTNAME");
  struct arglist * ports;
  struct arglist * port;
  char * port_s = emalloc(8);
  
  sprintf(port_s, "%d", portnum);
  if(!hostdata)return(NULL);
  ports = arg_get_value(hostdata, "PORTS");
  if(!ports)return(NULL);
  port = arg_get_value(ports, port_s);
  if(!port)return(NULL);
  return((char *)arg_get_value(port, "BANNER"));
}


char * plug_get_hostname(struct arglist * desc)
{
 struct arglist * hinfos = arg_get_value(desc, "HOSTNAME");
 if(hinfos)return((char*)arg_get_value(hinfos, "FQDN"));
 else return(NULL);
}

struct in_addr * plug_get_host_ip(struct arglist * desc)
{
 struct arglist * hinfos = arg_get_value(desc, "HOSTNAME");
 if(hinfos)return((struct in_addr*)arg_get_value(hinfos, "IP"));
 else return(NULL);
}


void post_hole(struct arglist * desc, int port, char * action)
{

 char * naction;
 char *t;
 struct arglist * hostdata;
 hostdata = arg_get_value(desc, "HOSTNAME");
 if(!action)return;
 naction = emalloc(strlen(action)+1);
 strncpy(naction, action, strlen(action));
 while((t=strchr(naction, '\n')))t[0]=';';
 
 printf("SERVER <|> HOLE <|> %s <|> %d:%s <|> SERVER\n", 
 	(char *)arg_get_value(hostdata, "FQDN"), port, naction);
 efree(naction);
 return;
} 


void post_info(struct arglist * desc, int port, char * action)
{
 char * naction,*t;
 struct arglist * hostdata;
 hostdata = arg_get_value(desc, "HOSTNAME");
 if(!action)return;
 
 naction = emalloc(strlen(action)+1);
 strncpy(naction, action, strlen(action));
 while((t=strchr(naction, '\n')))t[0]=';';
 
 printf("SERVER <|> INFO <|> %s <|> %d:%s <|> SERVER\n", 
 	(char *)arg_get_value(hostdata, "FQDN"), port, naction);
 efree(naction);
 return;
} 

void l_post_hole(struct arglist * desc, int port, char * action)
{
 struct arglist * ports;
 struct arglist * pport;
 struct arglist * report;
 char * sport = emalloc(20);
 char * comm;
 char * name = "General";
 
 if(!action)return;
 comm = emalloc(strlen(action)+1);
 strncpy(comm, action, strlen(action));
 
 ports = arg_get_value(desc, "PORTS");
 if(!ports && port == -1)
 {
  ports = emalloc(sizeof(struct arglist));
  arg_add_value(desc, "PORTS", ARG_ARGLIST, -1, ports);
 }
 if(port !=-1)
 {
 sprintf(sport, "%d", port);
 pport = arg_get_value(ports, sport);
 }
 else pport = arg_get_value(ports, name);

 if(!pport && port==-1){
 	pport = emalloc(sizeof(struct arglist));
 	arg_add_value(ports, name, ARG_ARGLIST, -1, pport);
 	arg_add_value(pport, "STATE", ARG_INT, sizeof(int),(void *)1);
 	}
 efree(sport);
 report = arg_get_value(pport, "REPORT");
 if(!report)
 {
  report = emalloc(sizeof(struct arglist));
  arg_add_value(pport, "REPORT", ARG_ARGLIST, -1, report);
 }
 while(report && report->next)report = report->next;
 arg_add_value(report, "HOLE", ARG_STRING, strlen(comm), comm);
 
 return;
} 




void l_post_info(struct arglist * desc, int port, char * action)
{
 struct arglist * ports;
 struct arglist * pport;
 struct arglist * report;
 char * sport = emalloc(20);
 char * comm;
 char * name = "General";
 
 if(!action)return;
 comm = emalloc(strlen(action)+1);
 strncpy(comm, action, strlen(action));
 
 ports = arg_get_value(desc, "PORTS");
 if(!ports && port == -1)
 {
  ports = emalloc(sizeof(struct arglist));
  arg_add_value(desc, "PORTS", ARG_ARGLIST, -1, ports);
 }
 if(port !=-1)
 {
 sprintf(sport, "%d", port);
 pport = arg_get_value(ports, sport);
 }
 else pport = arg_get_value(ports, name);
 if(!pport && port==-1){
 	pport = emalloc(sizeof(struct arglist));
 	arg_add_value(ports, name, ARG_ARGLIST, -1, pport);
 	arg_add_value(pport, "STATE", ARG_INT, sizeof(int),(void *)1);
 	}
 efree(sport);
 report = arg_get_value(pport, "INFO");
 if(!report)
 {
  report = emalloc(sizeof(struct arglist));
  arg_add_value(pport, "INFO", ARG_ARGLIST, -1, report);
 }
 while(report && report->next)report = report->next;
 arg_add_value(report, "INFO", ARG_STRING, strlen(comm), comm);
 
 return;
} 
 
char * get_preference(struct arglist *desc, char * name)
{
 struct arglist * prefs = arg_get_value(desc, "preferences");
 if(!prefs)return(NULL);
 return((char *)arg_get_value(prefs, name));
}
